;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;			CHRONICLES OF ATARIA				;
;		  by Maciej Kozlowski (Matzieq)			;
;		     matzieq@poczta.onet.pl				;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	processor 6502
	include vcs.h

	org $F000
	SEG.U VARS
	ORG $80

fineAdjustTable = fineAdjustBegin - %11110001 

Scanline			ds 1	;actual scanline
Room				ds 1	;actual room
Room_Address		ds 2	;pointer to room data
Room_Temp			ds 1	;used in getting data for floors
Knight_X			ds 1	;obvious
Knight_Y			ds 1

Knight_Frame_Address	ds 2
Knight_Sprite_Address	ds 2
Scanline_Temp		ds 1
Knight_State		ds 1	;bit 0: 1= knight's jumping/falling, 0= knight's standing
   					; bit 1: 0= knight's
					;normal, walking, jumping etc., 1= knight swings his sword
					;bit 2: 0=monsters are still active in this room
					;bit 3: 0= new room - need to create it
					;bit 4: 1= knight's defending
Temp 				ds 1 	;used for various temporary storage
Jump				ds 1	;jump lenght

Frame				ds 1	;knight's frame offset
Move				ds 1	;frequency of changing knight's animation frame
Monster_X			ds 4	;obvious: one byte for each monster
Monster_X_Boundary	ds 4	;used to make sure that monster moves only within one platform
Monster_Hp			ds 4	;monster's lives

Monster_Sprite_Address  ds 2	
Monster_Frame_Address	ds 2

Monster_Kind		ds 1	;type (speed) of monsters

Monster_Move		ds 1	;same as move - but for monsters

Lives				ds 1	;knight's hit points
Healing_Potions		ds 1	;spells in knight's
Fire_Balls			ds 1	;possession
Invulnerabilities		ds 1	

Monster_Anim		ds 1	;used to count frames
Monster_Frame  		ds 1	;same as frame, for monsters
Monster_Fights		ds 1	;indicator of monster attack (i was tired of all this
					;fiddling with knight state, so I wasted a whole byte...)
Monsters_Active		ds 1	;counter of monsters

Room_Platform_1		ds 5	;data for PF registers
Room_Platform_2		ds 5
Room_Platform_3		ds 5
Room_Platform_4		ds 5

Prize				ds 1	;what we're gonna get at the end

Spell_Kind			ds 1	;which spell is active
Spell_Duration		ds 1	;how long it will work
Spell_Dir			ds 1	;in which direction (fireball)
Fireball_Y			ds 1	

Monster_Magic		ds 1	
Monster_Missile_Y		ds 1
Fire_Pressed		ds 1
Which_Monster		ds 1
Monster_Fire_Duration	ds 1
Monster_Tel_Duration	ds 1
Monster_Spell_Dir		ds 1
Monster_Color		ds 1
Monster_Fire_Counter	ds 1
Monster_Tel_Counter	ds 1
Monster_Missile_Dir	ds 1
Missile_On_Which_Floor	ds 1
Game_On			ds 1
Death				ds 1
Sound_Kind			ds 1
Sound_Duration		ds 1
Sound_Pitch			ds 1
New_Sound			ds 1

Sound_Kind_2		ds 1
Sound_Duration_2		ds 1
Sound_Pitch_2		ds 1
New_Sound_2			ds 1
Reset_Pressed		ds 1
Select_Pressed		ds 1
;TOTAL - 89 bytes
	SEG 
	org $F000


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Start
	SEI					;clearing the memory
	CLD
	LDX #$FF
	TXS
	
	LDA #0
Clear
	STA 0,X
	DEX
	BNE Clear

Go	
	JSR Prepare_Vars
Video_Frame
	LDA #2					;VSYNC
	STA VSYNC
	STA WSYNC
	STA WSYNC
	STA WSYNC
	
	LDA #43					;set the timer so that we know, 
						;when the VBLANK ends
	STA TIM64T
	LDA #0
	STA VSYNC
	JSR Check_Game_Switches

	LDA #<Rooms				;load the pointer to the rooms data
	STA Room_Address
	LDA #>Rooms
	STA Room_Address+1
	
	LDA Room_Address			;load the actual room
	CLC
	ADC Room
	STA Room_Address
	LDA Room_Address+1
	ADC #0
	STA Room_Address+1
	LDA Knight_State
	AND #%00001000				;if the room is new
	BNE No_New_Room

	JSR New_Room				;initialize it			
No_New_Room
	LDA Game_On
	BEQ Wait
	JSR Move_Knight

Wait						;wait for VBLANK end
	LDA INTIM
	BNE Wait

	JSR Draw_Screen				;this routine draws the whole screen

	LDA #2
	STA WSYNC
	STA VBLANK
	LDA #35					;set the timer so that we know, when the VBLANK ends
	STA TIM64T
	JSR Sound_Routine
	LDA Game_On
	BEQ WaitOverscan
	JSR Monster_Jazz			;everything that the VCS would like to know about
						;the monsters
	JSR Magic_Routine

	;LDA Death
	;BEQ No_Screen_Fl
	;AND #%00111111
	;STA COLUBK
No_Screen_Fl
	LDA Lives
	BNE WaitOverscan
	
	LDA #0
	STA Game_On
	
WaitOverscan					;wait for Overscan end
	LDA INTIM
	BNE WaitOverscan
	STA WSYNC
	JMP Video_Frame				;and here we go again!

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;This does (almost) everything about knight's movement,
;animation, changing screens and giving prizes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Move_Knight
	LDA Knight_State
	AND #%00000010			;if the knight swings his sword, he can't move
	BNE Up
	LDA Knight_State
	AND #%00010000
	BNE Up

	LDA #%01000000			;joystick - left
	BIT SWCHA
	BNE No_Left
	LDA Knight_X
	CMP #10
	BEQ No_Left			;if he's so far on the left, he can't move either
	LDA #%00001000
	STA REFP0
	DEC Knight_X
	DEC Move			;frame counter
No_Left
	LDA #%10000000			;joystick - right
	BIT SWCHA
	BNE Up

	LDA Knight_X
	CMP #132			;and right boundary
	BEQ No_Right
	LDA #0
	STA REFP0
	INC Knight_X
	DEC Move
	JMP Up
No_Right
	LDA Monsters_Active		;but if he tries to escape to the right
					;we need to check if he did kill all the monsters
	BNE Up				
	CLC				;if it's true, exit to the next room
	LDA Room
	ADC #4
	STA Room
	LDA #30
	STA Knight_X
	LDA Knight_State
	AND #%11110111
	STA Knight_State
	LDA Prize				;give knight a spell
	CMP #1
	BNE No_Heal
	INC Healing_Potions
No_Heal
	CMP #2
	BNE No_Fireball
	INC Fire_Balls
No_Fireball
	CMP #3
	BNE No_Invb
	INC Invulnerabilities
No_Invb
Up
	LDA Knight_State
	AND #%00010000
	BEQ Nothing
	LDA Knight_State
	AND #%11101111
	STA Knight_State
	LDA #0
	STA Frame
Nothing
	LDA #%00010000			;joystick - up
	BIT SWCHA
	BNE No_Up
	LDA Knight_State
	AND #%00000001
	BEQ No_Up
	LDA Knight_State		;change knight's state to jump
	AND #%11111110
	STA Knight_State
	LDA #34
	STA Jump			;jump height
No_Up
	
	LDA #%00100000
	BIT SWCHA
	BNE No_Defend
	LDA Knight_State
	AND #%00000001
	BEQ No_Defend
	LDA Knight_State
	AND #%00000010
	BNE No_Defend	
	LDA Knight_State
	ORA #%00010000
	STA Knight_State
	JMP No_Fire
No_Defend
	
	LDA INPT4
	BMI No_Fire
	LDA Knight_State		;if player presses fire, and knight already swings
	AND #%00000010			;do nothing
	BNE No_Fire
	LDA Knight_State		;and if he jumps, he can't fight either
	AND #%00000001
	BEQ No_Fire		
	LDA Knight_State		;finally, if he can, we change his status to 'swinging'
	ORA #%00000010
	STA Knight_State
	LDA #120			;and initialize frame
	STA Frame
	LDA #6
	STA Move
	LDA #8
	STA Sound_Kind
	STA New_Sound
No_Fire

	LDA Jump			;if the knight isn't jumping
	BNE Jumping
	LDA #%10000000			;check if he's on the platform
	BIT CXP0FB
	BNE No_Falling
	DEC Knight_Y			;if not, he falls
	DEC Knight_Y
	LDA Knight_State
	AND #%11111110			;change his state to jumping/falling
	STA Knight_State
	JMP Go_On
No_Falling
	LDA Knight_State		;change knight's state to standing
	ORA #%00000001
	STA Knight_State
Go_On
	STA CXCLR
	JMP End_Moving_Knight
Jumping
	INC Knight_Y			;if knight jumps, he gets highter
	INC Knight_Y
	DEC Jump			;and jump lenght decreases
	DEC Jump
	STA CXCLR
End_Moving_Knight
	LDA Knight_State		;if the knight is jumping
	AND #%00000001			;we need to display adequate frame of his animation
	BNE Not_Jumping
	LDA #96
	STA Frame
	LDA #4				;it's here in case if the knight jumped when there was 0
	STA Move
	JMP End_Calculating_Frame
Not_Jumping
	LDA Knight_State
	AND #%00010000
	BEQ Not_Defending
	LDA #120
	STA Frame
	JMP End_Calculating_Frame
Not_Defending
	
	
	LDA Knight_State		;if the knight uses his sword, we need to animate him
	AND #%00000010			;when he stands still (normally, he's animated when
	BEQ No_Sword			;he walks)
	DEC Move
No_Sword
	LDA Move	
	BNE No_Changing_Frame		;see if we should change animation frame
	LDA #4	
	STA Move
	CLC 
	LDA Frame
	ADC #24
	STA Frame

No_Changing_Frame
	LDA Frame
	CMP #96					;if the walking animation is finished
	BNE End_Calculating_Frame		;start it again
	LDA #0
	STA Frame
	LDA Knight_State	
	AND #%11111101
	STA Knight_State
	JMP End_Calculating_Frame_The_Second
End_Calculating_Frame
	CMP #240				;and here sword animation
	BNE End_Calculating_Frame_The_Second
	LDA #0
	STA Frame
	LDA Knight_State	
	AND #%11111101
	STA Knight_State

End_Calculating_Frame_The_Second
	CLC
	LDA Knight_Frame_Address		;finally, calculating the actual
	ADC Frame				;frame of knight's animation
	STA Knight_Sprite_Address		;to display
	LDA Knight_Frame_Address+1
	ADC #0
	STA Knight_Sprite_Address+1

	RTS


fineAdjustBegin
	.byte #%01110000 ; Left 7 
	.byte #%01100000 ; Left 6
	.byte #%01010000 ; Left 5
	.byte #%01000000 ; Left 4
	.byte #%00110000 ; Left 3
	.byte #%00100000 ; Left 2
	.byte #%00010000 ; Left 1
	.byte #%00000000 ; No movement.
	.byte #%11110000 ; Right 1
	.byte #%11100000 ; Right 2
	.byte #%11010000 ; Right 3
	.byte #%11000000 ; Right 4
	.byte #%10110000 ; Right 5
	.byte #%10100000 ; Right 6
	.byte #%10010000 ; Right 7

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;This is the kernel. It must be optimized, 'cause it's totally space - wasting
;but it will be done later, after all elements of the game are on their places
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Draw_Screen
	STA WSYNC
	STA VBLANK

Knight_Pos

	
	SEC 
	LDA Knight_X	
Divideby15
	SBC #15 		; Waste the necessary amount of time dividing X-pos by 15!
	BCS Divideby15 		; 06/07 - 11/16/21/26/31/36/41/46/51/56/61/66

	TAY 			; At this point the value in A is -1 to -15. In this code I use a table
				; to quickly convert that value to the fine adjust value needed.
	LDA fineAdjustTable,Y 	; 13 -> Consume 5 cycles by guaranteeing we cross a page boundary
				; In your own code you may wish to consume only 4.
	STA HMP0 		; 17 Store the fine adjustment value.


	STA RESP0

	LDA Monster_Spell_Dir
	STA HMM1	

	STA WSYNC
	
	SEC 
	LDA Monster_X	
Divideby15_2
	SBC #15 		; Waste the necessary amount of time dividing X-pos by 15!
	BCS Divideby15_2	; 06/07 - 11/16/21/26/31/36/41/46/51/56/61/66

	TAY 			; At this point the value in A is -1 to -15. In this code I use a table
				; to quickly convert that value to the fine adjust value needed.
	LDA fineAdjustTable,Y 	; 13 -> Consume 5 cycles by guaranteeing we cross a page boundary
				; In your own code you may wish to consume only 4.
	STA HMP1 		; 17 Store the fine adjustment value.

	
	STA RESP1


	LDA Spell_Dir
	STA HMM0



	STA WSYNC
	STA HMOVE	

	LDA #%00100000
	STA NUSIZ0
	STA NUSIZ1
	LDX #40
	LDA Spell_Kind
	BEQ No_Spell_Working
	LDA Spell_Duration
	AND #%00001111
	TAX
	
No_Spell_Working
	LDA Lives
	BNE No_Zero_Lives
	LDX #0
No_Zero_Lives
	STX COLUP0
	LDA #182
	STA Scanline
	LDY #0
	
	STY PF0			;clear the playfield
	STY PF1				
	STY PF2		
	STY GRP0	
	STY GRP1		
	LDX #30			;amount of lines before we draw the playfield

	STA WSYNC

	LDA Missile_On_Which_Floor	
	CMP #%01100000
	BNE No_Monster_Missile_0	
	LDA Monster_Missile_Y
	BNE No_Monster_Missile_0
	LDA #2			
	STA RESMP1		
	LDA #0
	STA RESMP1
No_Monster_Missile_0

	LDA Spell_Duration	;if it's zero, means that no spell is active
	BNE Line_Floor_1	
	LDA #2			;so, reset missile position
	STA RESMP0		
	LDA #0
	STA RESMP0
	STA Spell_Kind



Line_Floor_1				
	STA WSYNC
	

	
	SEC			;+2 2
	LDA Scanline		;+2 4 
	
	;TYA

	SBC Knight_Y		;+3 7 hmmm.... I'm still trying to solve how does it work
	ADC #24			;+2 9 but it does!
	BCC End_Drawing_Knight	;+2 11

	TAY			;+2 13
	LDA (Knight_Sprite_Address),Y	;+5 18
	STA GRP0		;+3 21 Phew! It fits into horizontal blank!

	
End_Drawing_Knight
	LDA #0
	STA ENAM0
	LDA Spell_Duration
	BEQ No_FB
	LDA Spell_Kind
	BNE No_FB
	LDA Scanline
	AND #$FE
	CMP Fireball_Y
	BNE No_FB
	LDA #2
	STA ENAM0
No_FB


	DEC Scanline

	DEX
	STA WSYNC
	LDA Monster_Hp
	BEQ End_Drawing_Monster
	SEC			;+2 2
	
	TXA
	SBC #16			;+3 7 hmmm.... I'm still trying to solve how does it work
	ADC #18			;+2 9 but it does!
	BCC End_Drawing_Monster	;+2 11

	TAY			;+2 13
	LDA (Monster_Sprite_Address),Y	;+5 18
	STA GRP1		;+3 21 Phew! It fits into horizontal blank!
	
	
	;LDA Monster_Kind

	;STA COLUP1
	
End_Drawing_Monster
	LDA #0
	STA ENAM1
	LDA Monster_Missile_Y
	ORA #%00000001
	CMP Scanline
	BNE No_Monster_FB
	LDA Monster_Kind
	ORA #%00000010
	STA COLUP1
	STA ENAM1
No_Monster_FB

	DEC Scanline
	DEX
	BNE Line_Floor_1	
	DEC Scanline
	LDA #0
	STA HMP0
	STA HMM0
	STA HMM1
	STA WSYNC

	SEC 
	LDA Monster_X+1	
Divideby15_3
	SBC #15 		; Waste the necessary amount of time dividing X-pos by 15!
	BCS Divideby15_3	; 06/07 - 11/16/21/26/31/36/41/46/51/56/61/66

	TAY 			; At this point the value in A is -1 to -15. In this code I use a table
				; to quickly convert that value to the fine adjust value needed.
	LDA fineAdjustTable,Y 	; 13 -> Consume 5 cycles by guaranteeing we cross a page boundary
				; In your own code you may wish to consume only 4.
	STA HMP1 		; 17 Store the fine adjustment value.


	STA RESP1


	STA WSYNC
	STA HMOVE
	LDA #0
	STA GRP1
	LDA Room_Platform_1		;+3 3
	STA PF1				;+3 6
	LDA Room_Platform_1+1		;+3 9
	STA PF2				;+3 12
	LDX #4

Loop
	DEX
	BNE Loop
	LDA Room_Platform_1+2		;+3 41
	STA PF0				;+3 44
	LDA Room_Platform_1+3		;+3 47
	STA PF1				;+3 50
	LDA Room_Platform_1+4		;+3 53
	STA PF2				;+3 56
	LDA #0				;+2 58
	STA PF0				;+3 61
	NOP
	NOP
	;NOP
	STA PF2				;+3 75
	STA GRP0			;+3 64
	;STA GRP1			;+3 67
	STA PF1				;+2 70
	LDA Missile_On_Which_Floor	
	CMP #%01000000
	BNE No_Reset_Missile	
	LDA Monster_Missile_Y	
	BNE No_Reset_Missile	
	LDA #2			
	STA RESMP1		
	LDA #0
	STA RESMP1
No_Reset_Missile

	LDX #30				;+2 72
Line_Floor_2	
	
	STA WSYNC

	SEC			;+2 2
	LDY Scanline		;+2 4
	INY
	TYA

	SBC Knight_Y		;+3 7 hmmm.... I'm still trying to solve how does it work
	ADC #24			;+2 9 but it does!
	BCC End_Drawing_Knight_2	;+2 11

	TAY			;+2 13
	LDA (Knight_Sprite_Address),Y	;+5 18
	STA GRP0		;+3 21 Phew! It fits into horizontal blank!

	
	
End_Drawing_Knight_2
	LDA #0
	STA ENAM0
	LDA Spell_Duration
	BEQ No_FB_2
	LDA Spell_Kind
	BNE No_FB_2
	LDA Scanline
	AND #$FE
	CMP Fireball_Y
	BNE No_FB_2
	LDA #2
	STA ENAM0
No_FB_2
	DEC Scanline
	DEX
	STA WSYNC
	LDA Monster_Hp+1
	BEQ End_Drawing_Monster_2
	SEC			;+2 2
	
	TXA
	SBC #16			;+3 7 hmmm.... I'm still trying to solve how does it work
	ADC #18			;+2 9 but it does!
	BCC End_Drawing_Monster_2	;+2 11

	TAY			;+2 13
	LDA (Monster_Sprite_Address),Y	;+5 18
	STA GRP1		;+3 21 Phew! It fits into horizontal blank!

	;LDA Monster_Kind

	;TA COLUP1
	
	
End_Drawing_Monster_2
	LDA #0
	STA ENAM1
	LDA Monster_Missile_Y
	AND #$FE
	CMP Scanline
	BNE No_Monster_FB_2
		
	LDA Monster_Kind
	ORA #%00000010
	STA COLUP1
	STA ENAM1

No_Monster_FB_2

	DEC Scanline
	DEX
	BNE Line_Floor_2
	

	DEC Scanline

	STA WSYNC

	SEC 
	LDA Monster_X+2	
Divideby15_4
	SBC #15 		; Waste the necessary amount of time dividing X-pos by 15!
	BCS Divideby15_4	; 06/07 - 11/16/21/26/31/36/41/46/51/56/61/66

	TAY 			; At this point the value in A is -1 to -15. In this code I use a table
				; to quickly convert that value to the fine adjust value needed.
	LDA fineAdjustTable,Y 	; 13 -> Consume 5 cycles by guaranteeing we cross a page boundary
				; In your own code you may wish to consume only 4.
	STA HMP1 		; 17 Store the fine adjustment value.


	STA RESP1
	LDA #0
	STA HMP0


	STA WSYNC
	STA HMOVE


	LDA #0
	STA GRP1
	LDA Room_Platform_2		;+3 3
	STA PF1				;+3 6
	LDA Room_Platform_2+1		;+3 9
	STA PF2				;+3 12
	LDX #4

Loop_2
	DEX
	BNE Loop_2
	LDA Room_Platform_2+2		;+3 41
	STA PF0				;+3 44
	LDA Room_Platform_2+3		;+3 47
	STA PF1				;+3 50
	LDA Room_Platform_2+4		;+3 53
	STA PF2				;+3 56
	LDA #0				;+2 58
	STA PF0				;+3 61
	NOP
	NOP
	;NOP
	STA PF2				;+3 75
	STA GRP0			;+3 64
	;STA GRP1			;+3 67
	STA PF1				;+2 70
	LDA Missile_On_Which_Floor	
	CMP #%00100000
	BNE No_Reset_Missile_2	
	LDA Monster_Missile_Y	
	BNE No_Reset_Missile_2
	LDA #2			
	STA RESMP1		
	LDA #0
	STA RESMP1
No_Reset_Missile_2
	LDX #30		
Line_Floor_3
	
	STA WSYNC


	
	SEC			;+2 2
	LDY Scanline		;+2 4
	TYA

	SBC Knight_Y		;+3 7 hmmm.... I'm still trying to solve how does it work
	ADC #24		;+2 9 but it does!
	BCC End_Drawing_Knight_3	;+2 11
	TAY
	LDA (Knight_Sprite_Address),Y	;+5 18
	STA GRP0		;+3 21 Phew! It fits into horizontal blank!

	
	
End_Drawing_Knight_3
	LDA #0
	STA ENAM0
	LDA Spell_Duration
	BEQ No_FB_3
	LDA Spell_Kind
	BNE No_FB_3
	LDA Scanline
	AND #$FE
	CMP Fireball_Y
	BNE No_FB_3
	LDA #2
	STA ENAM0
No_FB_3
	DEC Scanline
	DEX
	STA WSYNC
	LDA Monster_Hp+2
	BEQ End_Drawing_Monster_3
	SEC			;+2 2
	
	TXA
	SBC #16			;+3 7 hmmm.... I'm still trying to solve how does it work
	ADC #18			;+2 9 but it does!
	BCC End_Drawing_Monster_3	;+2 11

	TAY			;+2 13
	LDA (Monster_Sprite_Address),Y	;+5 18
	STA GRP1		;+3 21 Phew! It fits into horizontal blank!
	
	;LDA Monster_Kind

	;STA COLUP1
	
	
End_Drawing_Monster_3
	LDA #0
	STA ENAM1
	LDA Scanline
	;AND #$FE
	CMP Monster_Missile_Y
	BNE No_Monster_FB_3
	
	LDA Monster_Kind
	ORA #%00000010
	STA COLUP1
	STA ENAM1
No_Monster_FB_3

	DEC Scanline
	DEX
	
	BNE Line_Floor_3
	
	DEC Scanline




	STA WSYNC
	SEC 
	LDA Monster_X+3
Divideby15_5
	SBC #15 		; Waste the necessary amount of time dividing X-pos by 15!
	BCS Divideby15_5	; 06/07 - 11/16/21/26/31/36/41/46/51/56/61/66

	TAY 			; At this point the value in A is -1 to -15. In this code I use a table
				; to quickly convert that value to the fine adjust value needed.
	LDA fineAdjustTable,Y 	; 13 -> Consume 5 cycles by guaranteeing we cross a page boundary
				; In your own code you may wish to consume only 4.
	STA HMP1 		; 17 Store the fine adjustment value.


	STA RESP1
	LDA #0
	STA HMP0


	STA WSYNC
	STA HMOVE

	LDA #0
	STA GRP1
	LDA Room_Platform_3		;+3 3
	STA PF1				;+3 6
	LDA Room_Platform_3+1		;+3 9
	STA PF2				;+3 12
	LDX #4

Loop_3
	DEX
	BNE Loop_3
	LDA Room_Platform_3+2		;+3 41
	STA PF0				;+3 44
	LDA Room_Platform_3+3		;+3 47
	STA PF1				;+3 50
	LDA Room_Platform_3+4		;+3 53
	STA PF2				;+3 56
	LDA #0				;+2 58
	STA PF0				;+3 61
	NOP
	NOP
	;NOP
	STA PF2				;+3 75
	STA GRP0			;+3 64
	;STA GRP1			;+3 67
	STA PF1				;+2 70
	LDA Missile_On_Which_Floor	
	
	BNE No_Reset_Missile_3	

	LDA Monster_Missile_Y	
	BNE No_Reset_Missile_3
	LDA #2			
	STA RESMP1		
	LDA #0
	STA RESMP1
No_Reset_Missile_3
	LDX #30				

Line_Floor_4
	
	STA WSYNC



	
	SEC			;+2 2
	LDY Scanline		;+2 4
	DEY
	TYA

	SBC Knight_Y		;+3 7 hmmm.... I'm still trying to solve how does it work
	ADC #24			;+2 9 but it does!
	BCC End_Drawing_Knight_4	;+2 11

	TAY			;+2 13
	LDA (Knight_Sprite_Address),Y	;+5 18
	STA GRP0		;+3 21 Phew! It fits into horizontal blank!

	
	
End_Drawing_Knight_4

	LDA #0
	STA ENAM0
	LDA Spell_Duration
	BEQ No_FB_4
	LDA Spell_Kind
	BNE No_FB_4
	LDA Scanline
	AND #$FE
	CMP Fireball_Y
	BNE No_FB_4
	LDA #2
	STA ENAM0
No_FB_4
	DEC Scanline
	DEX
	STA WSYNC
	LDA Monster_Hp+3
	BEQ End_Drawing_Monster_4
	SEC			;+2 2
	
	TXA
	SBC #16			;+3 7 hmmm.... I'm still trying to solve how does it work
	ADC #18			;+2 9 but it does!
	BCC End_Drawing_Monster_4	;+2 11

	TAY			;+2 13
	LDA (Monster_Sprite_Address),Y	;+5 18
	STA GRP1		;+3 21 Phew! It fits into horizontal blank!

	;LDA Monster_Kind

	;STA COLUP1
	
	
End_Drawing_Monster_4
	LDA #0
	STA ENAM1
	LDA Monster_Missile_Y
	AND #$FE
	CMP Scanline
	BNE No_Monster_FB_4
		
	LDA Monster_Kind
	ORA #%00000010
	STA COLUP1
	STA ENAM1

No_Monster_FB_4

	DEC Scanline
	DEX
	BNE Line_Floor_4
	

	DEC Scanline
	STA WSYNC

	LDA #0
	STA GRP1
	LDA Room_Platform_4		;+3 3
	STA PF1				;+3 6
	LDA Room_Platform_4+1		;+3 9
	STA PF2				;+3 12
	LDX #4

Loop_4
	DEX
	BNE Loop_4
	LDA Room_Platform_4+2		;+3 41
	STA PF0				;+3 44
	LDA Room_Platform_4+3		;+3 47
	STA PF1				;+3 50
	LDA Room_Platform_4+4		;+3 53
	STA PF2				;+3 56
	LDA #0				;+2 58
	STA PF0				;+3 61
	NOP
	NOP
	NOP
	STA PF2				;+3 75
	STA GRP0			;+3 64
	STA GRP1			;+3 67
	STA PF1				;+2 70

	DEC Scanline
	STA WSYNC
	STA RESP0
	LDX #0
	LDA Lives
	CMP #2
	BNE No_Two
	LDX #1
No_Two 
	CMP #3
	BNE No_Three
	STA NUSIZ0
	JMP Three

	
No_Three
	STX NUSIZ0
Three
	LDA #$33
	STA COLUP0
	LDX #5
Line_Lives
	STA WSYNC
	LDA Lives_Sprite-1,X
	STA GRP0
	DEC Scanline
	DEX
	BPL Line_Lives
	LDA #0
	STA GRP0
	DEC Scanline
	STA WSYNC
	LDX #0
	STX COLUP0
	STA RESP0
	LDY #$C5


	LDA Healing_Potions
	BNE Potions_Present
	LDY #0
Potions_Present
	CMP #2
	BNE No_Two_2
	LDX #1
No_Two_2
	CMP #3
	BNE No_Three_2
	STA NUSIZ0
	JMP Three_2

	
No_Three_2
	STX NUSIZ0
Three_2

	STY COLUP0
	LDX #5
Line_Balls
	STA WSYNC
	LDA Potion_Sprite-1,X
	STA GRP0
	DEC Scanline
	DEX
	BPL Line_Balls
	LDA #0
	STA GRP0
	DEC Scanline
	STA WSYNC
	LDX #0
	STX COLUP0
	STA RESP0
	LDY #30


	LDA Fire_Balls
	BNE Fireballs_Present
	LDY #0
Fireballs_Present
	CMP #2
	BNE No_Two_3
	LDX #1
No_Two_3
	CMP #3
	BNE No_Three_3
	STA NUSIZ0
	JMP Three_3

	
No_Three_3
	STX NUSIZ0
Three_3

	STY COLUP0
	LDX #5
Line_Potions
	STA WSYNC
	LDA Fireball_Sprite-1,X
	STA GRP0
	DEC Scanline
	DEX
	BPL Line_Potions
	LDA #0
	STA GRP0


	STA WSYNC
	LDX #0
	STX COLUP0
	STA RESP0
	LDY #$37


	LDA Invulnerabilities
	BNE Inv_Present
	LDY #0
Inv_Present
	CMP #2
	BNE No_Two_4
	LDX #1
No_Two_4
	CMP #3
	BNE No_Three_4
	STA NUSIZ0
	JMP Three_4

	
No_Three_4
	STX NUSIZ0
Three_4

	STY COLUP0
	LDX #5
Line_Inv
	STA WSYNC
	LDA Invulnerable_Sprite-1,X
	STA GRP0
	DEC Scanline
	DEX
	BPL Line_Inv
	LDA #0
	STA GRP0

	LDX Prize
	BEQ Line_End
	CLC
	LDA #5
Which_Prize
	ADC #5
	DEX
	BNE Which_Prize
	TAX
	LDY #5
	STA RESP0
	LDA #0
	STA NUSIZ0
	STY COLUP0
Line_Prize
	STA WSYNC
	LDA Lives_Sprite-1,X
	STA GRP0
	DEC Scanline
	DEX
	DEY
	BPL Line_Prize

	LDA #0
	STA GRP0
Line_End
	STA WSYNC
	DEC Scanline
	BNE Line_End
	RTS



Lives_Sprite

	.byte #%00011000
	.byte #%00111100
	.byte #%01111110
	.byte #%11100111
	.byte #%01100110

Potion_Sprite
	.byte #%00111100
	.byte #%01111110
	.byte #%01000010
	.byte #%00100100
	.byte #%00100100

Fireball_Sprite
	.byte #%00111100
	.byte #%01111110
	.byte #%11111111
	.byte #%01111110
	.byte #%00111100

Invulnerable_Sprite
	.byte #%11000011
	.byte #%01100110
	.byte #%00011000
	.byte #%01100110
	.byte #%11000011


Knight_Sprite
	.byte #%00000000	
	.byte #%00000000	
	.byte #%01100000
	.byte #%00000000	
	.byte #%01000000
	.byte #%00000000	
	.byte #%01000000
	.byte #%00000000	
	.byte #%01000000
	.byte #%00000000	
	.byte #%01101000
	.byte #%00000000	
	.byte #%01110100
	.byte #%00000000	
	.byte #%01100100
	.byte #%00000000	
	.byte #%00000100
	.byte #%00000000	
	.byte #%01100010
	.byte #%00000000	
	.byte #%01100010
	.byte #%00000000	
	.byte #%00000000
	.byte #%00000000

	.byte #%00000000	
	.byte #%00000000
	.byte #%01101100
	.byte #%00000000
	.byte #%01001000
	.byte #%00000000
	.byte #%00101000
	.byte #%00000000
	.byte #%01010000
	.byte #%00000000
	.byte #%01101000
	.byte #%00000000
	.byte #%01110100
	.byte #%00000000
	.byte #%01100100
	.byte #%00000000
	.byte #%00000100
	.byte #%00000000
	.byte #%01100010
	.byte #%00000000
	.byte #%01100010
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000

	.byte #%00000000	
	.byte #%00000000
	.byte #%11000011
	.byte #%00000000
	.byte #%10000100
	.byte #%00000000
	.byte #%01000100
	.byte #%00000000
	.byte #%00101000
	.byte #%00000000
	.byte #%00110100
	.byte #%00000000
	.byte #%00110100
	.byte #%00000000
	.byte #%00110010
	.byte #%00000000
	.byte #%00000010
	.byte #%00000000
	.byte #%00110010
	.byte #%00000000
	.byte #%00110010
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000

	.byte #%00000000	
	.byte #%00000000
	.byte #%00001100
	.byte #%00000000
	.byte #%10001000
	.byte #%00000000
	.byte #%11001000
	.byte #%00000000
	.byte #%00110000
	.byte #%00000000
	.byte #%00101100
	.byte #%00000000
	.byte #%01110100
	.byte #%00000000
	.byte #%01100100
	.byte #%00000000
	.byte #%00000100
	.byte #%00000000
	.byte #%01100010
	.byte #%00000000
	.byte #%01100010
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000

	.byte #%00000000	
	.byte #%00000000
	.byte #%11000000
	.byte #%00000000
	.byte #%10000110
	.byte #%00000000
	.byte #%01000100
	.byte #%00000000
	.byte #%01000010
	.byte #%00000000
	.byte #%01101100
	.byte #%00000000
	.byte #%01110100
	.byte #%00000000
	.byte #%01100100
	.byte #%00000000
	.byte #%00000100
	.byte #%00000000
	.byte #%01100010
	.byte #%00000000
	.byte #%01100010
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000

	.byte #%00000000	
	.byte #%00000000
	.byte #%01100000
	.byte #%00000000
	.byte #%01000000
	.byte #%00000000
	.byte #%01000000
	.byte #%00000000
	.byte #%01000000
	.byte #%00000000
	.byte #%01100000
	.byte #%00000000
	.byte #%01111100
	.byte #%00000000
	.byte #%01101000
	.byte #%00000000
	.byte #%00010000
	.byte #%00000000
	.byte #%01100000
	.byte #%00000000
	.byte #%11100000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000

	.byte #%00000000	
	.byte #%00000000
	.byte #%01100000
	.byte #%00000000
	.byte #%01000000
	.byte #%00000000
	.byte #%01000000
	.byte #%00000000
	.byte #%01000000
	.byte #%00000000
	.byte #%01101000
	.byte #%00000000
	.byte #%01110100
	.byte #%00000000
	.byte #%01100010
	.byte #%00000000
	.byte #%00000001
	.byte #%00000000
	.byte #%01100000
	.byte #%00000000
	.byte #%01100000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000

	.byte #%00000000	
	.byte #%00000000
	.byte #%01100000
	.byte #%00000000
	.byte #%01000000
	.byte #%00000000
	.byte #%01000000
	.byte #%00000000
	.byte #%01000000
	.byte #%00000000
	.byte #%01101100
	.byte #%00000000
	.byte #%01110011
	.byte #%00000000
	.byte #%00110000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #%00110000
	.byte #%00000000
	.byte #%00110000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000

	.byte #%00000000	
	.byte #%00000000
	.byte #%01100000
	.byte #%00000000
	.byte #%01000000
	.byte #%00000000
	.byte #%01000000
	.byte #%00000000
	.byte #%01000011
	.byte #%00000000
	.byte #%01101100
	.byte #%00000000
	.byte #%01110000
	.byte #%00000000
	.byte #%00110000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #%00110000
	.byte #%00000000
	.byte #%00110000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000

	.byte #%00000000	
	.byte #%00000000
	.byte #%01100000
	.byte #%00000000
	.byte #%01000011
	.byte #%00000000
	.byte #%00101100
	.byte #%00000000
	.byte #%01001000
	.byte #%00000000
	.byte #%01101000
	.byte #%00000000
	.byte #%01100000
	.byte #%00000000
	.byte #%00110000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #%00110000
	.byte #%00000000
	.byte #%00110000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000

Monster_Sprite
;sprite 1
	.byte #0
	.byte #%00000000
	.byte #%11101110
	.byte #%11101110
	.byte #%11001100
	.byte #%11001100
	.byte #%01110000
	.byte #%01110000
	.byte #%01111000
	.byte #%01111000
	.byte #%01100110
	.byte #%01100110
	.byte #%01110011
	.byte #%01110011
	.byte #%01110000
	.byte #%01110000
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%01111100
	.byte #%01111100
	.byte #%01111000
	.byte #%01111000
	.byte #%01110000
	.byte #%01110000
	.byte #%01111000
	.byte #%01111000
	.byte #%01100110
	.byte #%01100110
	.byte #%01110011
	.byte #%01110011
	.byte #%01110000
	.byte #%01110000
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%11101110
	.byte #%11101110
	.byte #%11001100
	.byte #%11001100
	.byte #%01110000
	.byte #%01110000
	.byte #%01111000
	.byte #%01111000
	.byte #%01100110
	.byte #%01100110
	.byte #%01110110
	.byte #%01110110
	.byte #%01110000
	.byte #%01110000
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%11101110
	.byte #%11101110
	.byte #%11001100
	.byte #%11001100
	.byte #%01110000
	.byte #%01110000
	.byte #%01111111
	.byte #%01111111
	.byte #%00110000
	.byte #%00110000
	.byte #%00111000
	.byte #%00111000
	.byte #%00111000
	.byte #%00111000
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%11101110
	.byte #%11101110
	.byte #%11011000
	.byte #%11011000
	.byte #%01111111
	.byte #%01111111
	.byte #%01111000
	.byte #%01111000
	.byte #%00110000
	.byte #%00110000
	.byte #%00111000
	.byte #%00111000
	.byte #%00111000
	.byte #%00111000
	.byte #0
	.byte #0

;sprite 2
	.byte #0
	.byte #%00000000
	.byte #%00111000
	.byte #%00111000
	.byte #%01101100
	.byte #%01101100
	.byte #%01000100
	.byte #%01000100
	.byte #%10000010
	.byte #%10000010
	.byte #%10000100
	.byte #%10000100
	.byte #%10001000
	.byte #%10001000
	.byte #%01110000
	.byte #%01110000
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #%00111100
	.byte #%00111100
	.byte #%00100100
	.byte #%00100100
	.byte #%01000010
	.byte #%01000010
	.byte #%10000100
	.byte #%10000100
	.byte #%01000100
	.byte #%01000100
	.byte #%00111000
	.byte #%00111000
	.byte #0
	.byte #0



	.byte #0
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #%00001000
	.byte #%00001000
	.byte #%00010100
	.byte #%00010100
	.byte #%00100010
	.byte #%00100010
	.byte #%01000100
	.byte #%01000100
	.byte #%01000110
	.byte #%01000110
	.byte #%10111011
	.byte #%10111011
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%00111100
	.byte #%00111100
	.byte #%01000010
	.byte #%01000010
	.byte #%10000001
	.byte #%10000001
	.byte #%10000001
	.byte #%10000001
	.byte #%10111101
	.byte #%10111101
	.byte #%11011011
	.byte #%11011011
	.byte #%11000011
	.byte #%11000011
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #%00011000
	.byte #%00011000
	.byte #%00100100
	.byte #%00100100
	.byte #%00011000
	.byte #%00011000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #0
	.byte #0

;sprite 3
	.byte #0
	.byte #%00000000
	.byte #%11101110
	.byte #%11101110
	.byte #%11001100
	.byte #%11001100
	.byte #%01110000
	.byte #%01110000
	.byte #%01111000
	.byte #%01111000
	.byte #%01100110
	.byte #%01100110
	.byte #%01110011
	.byte #%01110011
	.byte #%01110000
	.byte #%01110000
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%01111100
	.byte #%01111100
	.byte #%01111000
	.byte #%01111000
	.byte #%01110000
	.byte #%01110000
	.byte #%01111000
	.byte #%01111000
	.byte #%01100110
	.byte #%01100110
	.byte #%01110011
	.byte #%01110011
	.byte #%01110000
	.byte #%01110000
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%11101110
	.byte #%11101110
	.byte #%11001100
	.byte #%11001100
	.byte #%01110000
	.byte #%01110000
	.byte #%01111000
	.byte #%01111000
	.byte #%01100110
	.byte #%01100110
	.byte #%01110110
	.byte #%01110110
	.byte #%01110000
	.byte #%01110000
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%11101110
	.byte #%11101110
	.byte #%11001100
	.byte #%11001100
	.byte #%01110000
	.byte #%01110000
	.byte #%01111111
	.byte #%01111111
	.byte #%00110000
	.byte #%00110000
	.byte #%00111000
	.byte #%00111000
	.byte #%00111000
	.byte #%00111000
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%11101110
	.byte #%11101110
	.byte #%11011000
	.byte #%11011000
	.byte #%01111111
	.byte #%01111111
	.byte #%01111000
	.byte #%01111000
	.byte #%00110000
	.byte #%00110000
	.byte #%00111000
	.byte #%00111000
	.byte #%00111000
	.byte #%00111000
	.byte #0
	.byte #0
;sprite 4
	.byte #0
	.byte #%00000000
	.byte #%11111111
	.byte #%11111111
	.byte #%01111110
	.byte #%01111110
	.byte #%01111110
	.byte #%01111110
	.byte #%00111100
	.byte #%00111100
	.byte #%00111100
	.byte #%00111100
	.byte #%00011000
	.byte #%00011000
	.byte #%00000000
	.byte #%00000000
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%11111111
	.byte #%11111111
	.byte #%00111110
	.byte #%00111110
	.byte #%00011110
	.byte #%00011110
	.byte #%00011100
	.byte #%00011100
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #%00000000
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%11111111
	.byte #%11111111
	.byte #%11111111
	.byte #%11111111
	.byte #%01111110
	.byte #%01111110
	.byte #%01111100
	.byte #%01111100
	.byte #%01111110
	.byte #%01111110
	.byte #%00111100
	.byte #%00111100
	.byte #%00011000
	.byte #%00011000
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%11111111
	.byte #%11111111
	.byte #%11111111
	.byte #%11111111
	.byte #%01111101
	.byte #%01110000
	.byte #%01110000
	.byte #%01111001
	.byte #%01111110
	.byte #%01111110
	.byte #%00111100
	.byte #%00111100
	.byte #%00001100
	.byte #%00001100
	.byte #0
	.byte #0

	.byte #0
	.byte #%00000000
	.byte #%11111111
	.byte #%11111111
	.byte #%11111111
	.byte #%11111111
	.byte #%01111110
	.byte #%01111110
	.byte #%00111110
	.byte #%00111110
	.byte #%00011110
	.byte #%00011110
	.byte #%00001100
	.byte #%00001100
	.byte #%00000000
	.byte #%00000000
	.byte #0
	.byte #0


New_Room
	LDY #2				;this is reading from byte 3 of the room data
	LDA (Room_Address),Y		;and calculating initial X of the monsters
	STA Room_Temp
	AND #%11000000
	LSR
	LSR
	LSR
	LSR
	LSR
	LSR
	TAX
	INX
	CLC
	LDA #0
Position_1_Monster
	ADC #32
	DEX
	BNE Position_1_Monster
	STA Monster_X

	LDA Room_Temp
	AND #%00110000
	LSR
	LSR
	LSR
	LSR
	TAX
	INX
	CLC 
	LDA #0
Position_2_Monster
	ADC #32
	DEX
	BNE Position_2_Monster
	STA Monster_X+1


	LDA Room_Temp
	AND #%00001100
	LSR
	LSR

	TAX
	INX
	CLC 
	LDA #0
Position_3_Monster
	ADC #32
	DEX
	BNE Position_3_Monster
	STA Monster_X+2


	LDA Room_Temp
	AND #%00000011

	TAX
	INX
	CLC 
	LDA #0
Position_4_Monster
	ADC #32
	DEX
	BNE Position_4_Monster
	STA Monster_X+3

	LDY #3				;here, we read the kind of the monsters at the room
	LDA (Room_Address),Y
	STA Monster_Magic
	AND #%11100000
	STA Monster_Tel_Counter
	LSR
	LSR
	LSR
	LSR
	STA COLUP1
	STA Monster_Kind
	
	STA Monster_Move
	

	LDA #16
	STA Monster_X_Boundary
	STA Monster_X_Boundary+1
	STA Monster_X_Boundary+2
	STA Monster_X_Boundary+3
	LDA #3
	STA Monster_Hp
	STA Monster_Hp+1
	STA Monster_Hp+2
	STA Monster_Hp+3
	LDA Knight_State
	ORA #%00001000
	STA Knight_State
	LDA #4
	STA Monsters_Active
	LDA (Room_Address),Y
	;AND #%00011100
	;LSR
	;LSR
	STA Monster_Magic
	;LDA (Room_Address),Y
	AND #%00000011
	STA Prize
;LOADING ROOM DATA

	LDY #0

	LDA (Room_Address),Y		;now, checking the first floor of the room
	STA Room_Temp		
	LDY #0
	LDA Room_Temp			;+2 (2)	first platform
	AND #%10000000			;+2 (4)
	BEQ Platform_2_1		;+2 (6)	
	LDY #$FF

Platform_2_1
	STY Room_Platform_1				;+3 (9)
	LDY #0
	LDA Room_Temp			;+3 (12) second platform
	AND #%01000000			;+2 (14)
	BEQ Platform_3_1		;+2 (16) etc.
	LDY #$FF		

Platform_3_1
	STY Room_Platform_1+1			;+3 (19)
	LDY #0				;+2 (24)
	LDX #0
	LDA Room_Temp			;+3 (27)
	AND #%00100000			;+2 (29)
	BEQ Platform_4_1		;+2 (31)
	LDY #$FF			;+2 (33)
	LDX #$F0
Platform_4_1
	STY Room_Platform_1+2		;+3 (38)
	STX Room_Platform_1+3
	LDX #0				;+2 (35)
	LDA Room_Temp			;+3 (41)
	AND #%00010000			;+2 (43)
	BEQ End_Floor_1			;+2 (45)
	LDA Room_Platform_1+3 
	ORA #%00001111
	STA Room_Platform_1+3
	LDX #$0F			;+2 (49)
End_Floor_1

	STX Room_Platform_1+4			;+3 (56)


	LDY #0

	LDA (Room_Address),Y		;now, checking the first floor of the room
	STA Room_Temp		
	LDY #0
	LDA Room_Temp			;+2 (2)	first platform
	AND #%00001000			;+2 (4)
	BEQ Platform_2_2		;+2 (6)	
	LDY #$FF

Platform_2_2
	STY Room_Platform_2				;+3 (9)
	LDY #0
	LDA Room_Temp			;+3 (12) second platform
	AND #%00000100			;+2 (14)
	BEQ Platform_3_2		;+2 (16) etc.
	LDY #$FF		

Platform_3_2
	STY Room_Platform_2+1			;+3 (19)
	LDY #0				;+2 (24)
	LDX #0
	LDA Room_Temp			;+3 (27)
	AND #%00000010			;+2 (29)
	BEQ Platform_4_2		;+2 (31)
	LDY #$FF			;+2 (33)
	LDX #$F0
Platform_4_2
	STY Room_Platform_2+2		;+3 (38)
	STX Room_Platform_2+3
	LDX #0				;+2 (35)
	LDA Room_Temp			;+3 (41)
	AND #%00000001			;+2 (43)
	BEQ End_Floor_2			;+2 (45)
	LDA Room_Platform_2+3 
	ORA #%00001111
	STA Room_Platform_2+3
	LDX #$0F			;+2 (49)
End_Floor_2

	STX Room_Platform_2+4			;+3 (56)



	LDY #1

	LDA (Room_Address),Y		;now, checking the first floor of the room
	STA Room_Temp		
	LDY #0
	LDA Room_Temp			;+2 (2)	first platform
	AND #%10000000			;+2 (4)
	BEQ Platform_2_3		;+2 (6)	
	LDY #$FF

Platform_2_3
	STY Room_Platform_3				;+3 (9)
	LDY #0
	LDA Room_Temp			;+3 (12) second platform
	AND #%01000000			;+2 (14)
	BEQ Platform_3_3		;+2 (16) etc.
	LDY #$FF		

Platform_3_3
	STY Room_Platform_3+1			;+3 (19)
	LDY #0				;+2 (24)
	LDX #0
	LDA Room_Temp			;+3 (27)
	AND #%00100000			;+2 (29)
	BEQ Platform_4_3		;+2 (31)
	LDY #$FF			;+2 (33)
	LDX #$F0
Platform_4_3
	STY Room_Platform_3+2		;+3 (38)
	STX Room_Platform_3+3
	LDX #0				;+2 (35)
	LDA Room_Temp			;+3 (41)
	AND #%00010000			;+2 (43)
	BEQ End_Floor_3			;+2 (45)
	LDA Room_Platform_3+3 
	ORA #%00001111
	STA Room_Platform_3+3
	LDX #$0F			;+2 (49)
End_Floor_3

	STX Room_Platform_3+4			;+3 (56)

	LDY #1

	LDA (Room_Address),Y		;now, checking the first floor of the room
	STA Room_Temp		
	LDY #0
	LDA Room_Temp			;+2 (2)	first platform
	AND #%00001000			;+2 (4)
	BEQ Platform_2_4		;+2 (6)	
	LDY #$FF

Platform_2_4
	STY Room_Platform_4		;+3 (9)
	LDY #0
	LDA Room_Temp			;+3 (12) second platform
	AND #%00000100			;+2 (14)
	BEQ Platform_3_4		;+2 (16) etc.
	LDY #$FF		

Platform_3_4
	STY Room_Platform_4+1			;+3 (19)
	LDY #0				;+2 (24)
	LDX #0
	LDA Room_Temp			;+3 (27)
	AND #%00000010			;+2 (29)
	BEQ Platform_4_4		;+2 (31)
	LDY #$FF			;+2 (33)
	LDX #$F0
Platform_4_4
	STY Room_Platform_4+2		;+3 (38)
	STX Room_Platform_4+3
	LDX #0				;+2 (35)
	LDA Room_Temp			;+3 (41)
	AND #%00000001			;+2 (43)
	BEQ End_Floor_4			;+2 (45)
	LDA Room_Platform_4+3 
	ORA #%00001111
	STA Room_Platform_4+3
	LDX #$0F			;+2 (49)
End_Floor_4

	STX Room_Platform_4+4		

	LDA #<Monster_Sprite			;load the pointer to the monster's graphics
	STA Monster_Frame_Address
	LDA #>Monster_Sprite
	STA Monster_Frame_Address+1

	LDA Monster_Kind
	LSR
	LSR
	AND #%00000011
	BEQ No_Sprite_Change
	TAX

	
Counting_Offset
	CLC
	LDA Monster_Frame_Address
	ADC #90
	STA Monster_Frame_Address
	LDA Monster_Frame_Address+1
	ADC #0
	STA Monster_Frame_Address+1
	DEX 
	BNE Counting_Offset
	
No_Sprite_Change	
	
	RTS


Monster_Jazz
	SEC
	LDA Knight_Y			;determining on which floor the knight is
	SBC #54
	LSR				;floor height is 32
	LSR				;so when we trash the right five bytes
	LSR
	LSR
	LSR
	STA Temp			;and turn it around
	SEC
	LDA #$03			;we would have it
	SBC Temp
	STA Temp
	STA Which_Monster
	TAX
	DEC Monster_Move		;now, look if the monsters should move
	BNE No_Monster_Move		;if not - no move
	LDA Monster_Fights		;check if the monster is now attacking
	BEQ Normal_Animation		;if not - animate it normally

	CLC				;increase monster's frame
	LDA Monster_Frame
	ADC #18
	STA Monster_Frame
	
	INC Monster_Anim		;and animation lenght counter
	LDA Monster_Anim
	CMP #4				;if we reached end of the animation
	BNE Continue
	
	LDA #0				;reset animation
	STA Monster_Anim		;end monster's attack
	STA Monster_Fights
	STA Monster_Frame		;reset frame counter
	LDA Monster_Kind
	STA Monster_Move		;and begin the normal animation
	
	JMP Continue	

Normal_Animation
	LDA Monster_Anim		;normal animation - if monster anim is zero display first
	BNE Not_Zero			;frame, else second frame
	INC Monster_Anim
	LDA #0
	STA Monster_Frame
	JMP Continue
Not_Zero
	DEC Monster_Anim
	LDA #18
	STA Monster_Frame
Continue
	LDA Monster_Fights		;if the monster isn't attacking
	BNE Fighting_Move

	LDA Monster_Kind		;he moves at normal speed
	STA Monster_Move
	JMP No_Fighting
Fighting_Move
	LDA Monster_Kind		;if he is, he moves 4 times slower
	ASL
	ASL
	STA Monster_Move
No_Fighting
	LDY #%00001000
	LDA Monster_X,X			;check monster's position
	SEC
	SBC #8
	CMP Knight_X			;if his X is lower than knights
	BPL No_Lower
	LDA Monster_X_Boundary,X	;and if he didn't go out of the platform
	CMP #30
	BEQ No_Lower
	INC Monster_X,X			;move him towards the knight
	INC Monster_X,X
	INC Monster_X_Boundary,X
	INC Monster_X_Boundary,X
	LDY #0				;and don't reflect his image
No_Lower
	LDA Monster_X_Boundary,X	;else, move him in opposite direction
	CMP #0				;and reflect his image
	BEQ No_Monster_Move
	DEC Monster_X,X
	DEC Monster_X_Boundary,X

	STY REFP1

No_Monster_Move
	LDA Monster_Hp,X		;if his HP are 0, monster doesn't fight
	BEQ No_Fight
	LDA #%10000000			;if HP are present, we check if monster collides 
	BIT CXPPMM			;with knight
	BEQ No_Fight
	LDA Monster_Fights		;if he does, we check if he isn't already attacking
	BNE Check_Monster
	;LDA #1
	STA Monster_Anim		;and NOW we make him attack!
	INC Monster_Fights
	CLC
	LDA Monster_Kind		;store animation speed
	ASL
	ASL
	STA Monster_Move		;and frame counter
	LDA #18
	STA Monster_Frame
Check_Monster
	LDA Monster_Frame		;if the monster is on his second frame
	CMP #54				;of attack animation
	BNE Check_Knight		;and knight doesn't block
	LDA #1
	STA New_Sound
	STA Sound_Kind
	LDA Knight_State
	AND #%00010000
	BNE Check_Knight		;he loses one life
	LDA Spell_Kind	
	BNE Check_Knight
	DEC Lives
	LDA Monster_Frame
	CLC			
	ADC #18				;and monster's frame increases
	STA Monster_Frame
	INC Monster_Anim
	LDA #14
	STA Sound_Kind
	STA New_Sound
Check_Knight
	LDA Frame			;if knight is on his appropiate frame
	CMP #216
	BNE No_Fight			;he hits the monster
	
	DEC Monster_Hp,X
	BNE No_Disable
	DEC Monsters_Active		;if he kills a monster, number of monsters decreases
	LDA #0
	STA Monster_Fights
	LDA #$FF
	STA Monster_Fire_Duration
	
	LDA #1
	STA Monster_Move
	LDA Frame			;and we go to the next frame
No_Disable
	CLC
	ADC #24
	STA Frame
	LDA #10
	STA New_Sound_2
	STA Sound_Kind_2
	
No_Fight
	CLC
	LDA Monster_Frame_Address	;and now, calculating pointer for displaying monsters
	ADC Monster_Frame		;on the screen
	STA Monster_Sprite_Address
	LDA Monster_Frame_Address+1
	ADC #0
	STA Monster_Sprite_Address+1

	LDA Knight_Y 			;checking if the knight doesn't fall out of the screen
	CMP #80
	BPL Nothing_2
	CMP #70 
	BMI Nothing_2
	LDA #186
	STA Knight_Y
	LDA #10
	STA Knight_X
	DEC Lives
Nothing_2
	LDA Knight_State		;if he stands on the ground, he must have an
	AND #%00000001			;appropiate Y
	BEQ Nothing_3			;otherwise, from time to time we would observe
	LDA Knight_Y			;him hanging by the head on the platform or other 
	AND #%00011111			;dumb things
	CMP #16
	BPL Nothing_3
	LDA Knight_Y
	AND #%11100000
	CLC
	ADC #18
	STA Knight_Y
Nothing_3	
	LDA #%10000000
	BIT CXM0P
	BEQ No_Hit_By_Fireball
	SEC
	LDA Fireball_Y
	SBC #54
	LSR
	LSR
	LSR
	LSR
	LSR
	STA Temp			;and turn it around
	SEC
	LDA #$03			;we would have it
	SBC Temp
	STA Temp 
	TAX
	LDA #0
	STA Spell_Duration
	LDA Monster_Hp,X
	BEQ No_Hit_By_Fireball
	LDA #0
	STA Monster_Hp,X
	DEC Monsters_Active
	
No_Hit_By_Fireball

	LDA #%10000000
	BIT CXM1P
	BEQ No_Hit_By_Monster_Fire
	LDA Spell_Kind	
	BNE No_Hit_By_Monster_Fire
	LDA #0
	STA Monster_Missile_Y
	LDA #255
	STA Monster_Fire_Duration
	LDA #4
	STA Sound_Kind
	STA New_Sound
	DEC Lives
	
	
No_Hit_By_Monster_Fire

	RTS

Magic_Routine

	LDA Spell_Duration			;if any spell is present on the screen
	BNE No_Magic				;knight cannot do magic
	LDA Fire_Balls				;if he hasn't any fireballs
	BEQ No_Right_2				;he can't shoot
	LDA #%00000100
	BIT SWCHA					;right joystick right
	BNE No_Left_2
	DEC Fire_Balls
	LDA #0					;which spell is cast
	STA Spell_Kind
	
	LDA #$20					;right direction
	STA Spell_Dir			
	LDA #80					;lenght
	STA Spell_Duration
	SEC
	LDA Knight_Y				;appropiate Y
	SBC #16
	STA Fireball_Y
No_Left_2
	LDA #%00001000				;the same for the left
	BIT SWCHA
	BNE No_Right_2
	DEC Fire_Balls
	LDA #0
	STA Spell_Kind

	LDA #$E0
	STA Spell_Dir
	LDA #80
	STA Spell_Duration
	SEC
	LDA Knight_Y	
	SBC #16
	STA Fireball_Y
No_Right_2
	LDA INPT5					;if right joystick's fire is pressed
	BMI No_Fire_2				
	LDA Fire_Pressed				;but not constantly, if it's being held do nothing
	BNE No_Fire_3				;add to lives, if they're not above 3
	LDA Lives 
	CMP #3
	BEQ No_Fire_3
	INC Fire_Pressed
	LDA Healing_Potions
	BEQ No_Fire_2
	DEC Healing_Potions
	INC Lives
	LDA #13
	STA Sound_Kind
	STA New_Sound
	JMP No_Fire_3
No_Fire_2
	LDA #0				
	STA Fire_Pressed
No_Fire_3
	LDA #%00000010
	BIT SWCHA
	BNE No_Down_2
	LDA Invulnerabilities
	BEQ No_Down_2
	DEC Spell_Duration
	LDA #1
	STA Spell_Kind
	DEC Invulnerabilities
No_Down_2
	JMP After_Magic
No_Magic
	DEC Spell_Duration

After_Magic
	LDA Knight_Y
	SBC #54
	AND #%11100000

	STA Missile_On_Which_Floor
	LDA Monster_Magic
	AND #%00010000
	BEQ No_Monster_Fire_Spell	
	LDX Which_Monster
	LDA Monster_Hp,X
	BEQ No_Monster_Fire_Spell
	LDA Monster_Missile_Y
	BNE No_Monster_Fire
	DEC Monster_Fire_Duration
	BNE Check_Invisibility
	LDA #$FF
	STA Monster_Fire_Duration
	SEC
	

	LDA Knight_Y
	AND #%11100000
	CLC
	ADC #7	
	STA Monster_Missile_Y
	STA Sound_Kind_2
	STA New_Sound_2
	LDA #160
	STA Monster_Fire_Duration
	LDY #$10
	LDA Knight_X
	CLC
	ADC #7
	CMP Monster_X,X
	BMI Other_Dir
	LDY #$F0
Other_Dir
	STY Monster_Spell_Dir
	JMP Check_Invisibility
No_Monster_Fire
	DEC Monster_Fire_Duration
	BEQ No_Monster_Fire_Spell
	JMP Check_Invisibility
No_Monster_Fire_Spell
	LDA #0
	STA Monster_Missile_Y

	
	
Check_Invisibility
	LDA Monster_Magic
	AND #%00001000
	BEQ No_Invisible
	LDA Monster_Fights
	BEQ Less
	LDA Monster_Kind
	STA COLUP1
	JMP No_Invisible
Less
	LDA #0
	STA COLUP1
No_Invisible

	LDA Monster_Magic
	AND #%00000100
	BEQ No_Teleport
	DEC Monster_Tel_Duration
	BNE No_Teleport
	LDA Monster_Tel_Counter
	
	STA Monster_Tel_Duration

	LDX Which_Monster
	LDA Knight_X
	CLC
	ADC #7
	STA Monster_X,X
No_Teleport
	RTS


Prepare_Vars


	LDA #<Knight_Sprite			;load the pointer to the knight's graphics
	STA Knight_Frame_Address
	LDA #>Knight_Sprite
	STA Knight_Frame_Address+1

	LDA #<Monster_Sprite			;load the pointer to the monster's graphics
	STA Monster_Frame_Address
	LDA #>Monster_Sprite
	STA Monster_Frame_Address+1
	LDA #0
	STA Room
	LDA #20
	STA COLUPF
	LDA #30
	STA Knight_X
	;LDA #40

	;STA COLUP0
	;LDA #50
	;STA COLUP1
	LDA #180
	STA Knight_Y

	LDA #3
	STA Move
	STA Lives
	LDA #1
	STA Healing_Potions
	;STA Fire_Balls
	;STA Invulnerabilities
	LDA #%00000100
	STA Knight_State
	;STA CTRLPF
	RTS



Sound_Routine
	LDA Game_On
	BEQ No_Fire_Signal
	LDA Sound_Kind
	CMP #13
	BNE Check_1	
	DEC Sound_Pitch
	DEC Sound_Pitch
Check_1
	CMP #1
	BEQ No_Decrease_Pitch

	INC Sound_Pitch
No_Decrease_Pitch
	LDA Sound_Duration
	BMI No_Sound
	DEC Sound_Duration

	JMP Sound_Jazz
No_Sound
	LDA #0
	

		

Sound_Jazz

	STA AUDV1
	LDA Sound_Pitch
	STA AUDF1
	
	LDA New_Sound
	BEQ No_New_Sound
	LDA Sound_Kind
	STA AUDC1
	LDA #0
	STA Sound_Pitch
	;STA AUDF1
	LDA #15
	STA Sound_Duration
	STA AUDV1
	LDA #0
	STA New_Sound
No_New_Sound

	INC Sound_Pitch_2
	LDA Sound_Duration_2
	BMI No_Sound_2
	DEC Sound_Duration_2

	JMP Sound_Jazz_2
No_Sound_2
	LDA #0
	
Sound_Jazz_2
	STA AUDV0
	LDA Sound_Pitch_2
	STA AUDF0
	
	LDA New_Sound_2
	BEQ No_New_Sound_2
	LDA Sound_Kind_2
	STA AUDC0
	LDA #0
	STA Sound_Pitch_2
	;STA AUDF1
	LDA #15
	STA Sound_Duration_2
	STA AUDV0
	LDA #0
	STA New_Sound_2
No_New_Sound_2

	LDA Monster_Magic
	AND #%00010000
	BEQ No_Fire_Signal
	LDA Monster_Missile_Y
	BNE No_Fire_Signal
	LDA Monster_Fire_Duration
	AND #%11100010
	BNE No_Fire_Signal
	LDA #15
	STA AUDV0
	LDA #7
	STA AUDC0
	STA AUDF0	
No_Fire_Signal
	

	RTS

Check_Game_Switches

	LDA SWCHB		;on reset - start the game
	AND #%00000001
	BNE NoReset
	LDA Reset_Pressed
	BNE Check_Select
	LDA #1
	STA Reset_Pressed
	LDA Lives
	BNE No_Start
	JMP Go
No_Start
	LDA #1
	STA Game_On
NoReset
	LDA #0
	STA Reset_Pressed
Check_Select
	LDA SWCHB		
	AND #%00000010
	BNE No_Select
	LDA Select_Pressed
	BNE End_Checking
	LDA Game_On
	BNE End_Checking
	LDA #1
	STA Select_Pressed
	
	LDA Room
	CMP #60
	BPL End_Checking
	CLC
	ADC #16
	STA Room
	LDA #0	
	STA Knight_State
	JMP End_Checking
No_Select
	LDA #0
	STA Select_Pressed
End_Checking

	LDA Room
	CMP #120
	BMI No_End
	JMP Start
No_End
	RTS


Rooms

	.byte #%10010101		;left 4 bits are the first floor, right are the second
	.byte #%10011011		;and here left are the third, and right are the fourth
					;each floor consists of four platforms
					;and if the bit is 1 it means the platform is 'on'
	.byte #%00011110		;on which platform of each floor monster is present
	.byte #%11100000		;Left three - kind of monsters (speed), next three - 
					;monster's magic power, last two - level prize

	.byte #%01011000		;room 2
	.byte #%01001001

	.byte #%11000111		
	.byte #%11000001		

	.byte #%10110110		;room 3
	.byte #%10011111		
				
	.byte #%11011110		
	.byte #%10100000		

	.byte #%11010010		;room 4
	.byte #%01001000

	.byte #%01100100		
	.byte #%10100000

	.byte #%10010110		;room 5
	.byte #%01011000

	.byte #%00100100		
	.byte #%10000000	

	.byte #%10000001		;room 6
	.byte #%00100111

	.byte #%00111001		
	.byte #%10000010

	.byte #%00011110		;room 7
	.byte #%00011110

	.byte #%11101110		
	.byte #%01100000	

	.byte #%01000010		;room 8
	.byte #%11010010

	.byte #%01101110		
	.byte #%01100000	

	.byte #%11100110		;room 9
	.byte #%11011110

	.byte #%10010101		
	.byte #%01000011	


	.byte #%00100100		;room 10
	.byte #%10000011

	.byte #%10010010		
	.byte #%01000000	

	.byte #%01100010		;room 11
	.byte #%00111111

	.byte #%01101100		
	.byte #%01000001

	.byte #%00010010		;room 12
	.byte #%01001000

	.byte #%11100100		
	.byte #%11110010		

	.byte #%10000100		;room 13
	.byte #%00100001

	.byte #%00011011		
	.byte #%11010000
	
	.byte #%00100001		;room 14
	.byte #%01101100

	.byte #%10110101		
	.byte #%10110001

	.byte #%10001100		;room 15
	.byte #%10101101

	.byte #%00001001		
	.byte #%10110010

	.byte #%11011101		;room 16
	.byte #%11011111

	.byte #%00000011		
	.byte #%10010011

	.byte #%01011010		;room 17
	.byte #%01011010

	.byte #%01100110		
	.byte #%10010000

	.byte #%10000111		;room 18
	.byte #%10000110

	.byte #%00100010		
	.byte #%01110000

	.byte #%01011011		;room 19
	.byte #%01101000

	.byte #%01101000		
	.byte #%01110001

	.byte #%01001000		;room 20
	.byte #%00010010

	.byte #%01001110		
	.byte #%01010000

	.byte #%11100001		;room 21
	.byte #%11100001

	.byte #%01111011		
	.byte #%01010000

	.byte #%00100010		;room 22
	.byte #%00101101

	.byte #%10101011		
	.byte #%01010001

	.byte #%00011110		;room 23
	.byte #%01101000

	.byte #%11100100		
	.byte #%01010000

	.byte #%11100111		;room 24
	.byte #%11100111

	.byte #%10011001		
	.byte #%11111011

	.byte #%01101010		;room 25
	.byte #%11001011

	.byte #%10000111		
	.byte #%11011000

	.byte #%11100111		;room 26
	.byte #%01111000

	.byte #%00011000		
	.byte #%10111000

	.byte #%11100111		;room 27
	.byte #%01101111

	.byte #%00111011			
	.byte #%10011001

	.byte #%00010010		;room 28
	.byte #%01111000

	.byte #%11101100		
	.byte #%10011000

	.byte #%11100111		;room 29
	.byte #%11000011

	.byte #%01100010		
	.byte #%01111000

	.byte #%11100100		;room 30
	.byte #%10101001

	.byte #%00011011		
	.byte #%01110110


	org $FFFC
	.word Start
	.word Start